@@ -6,32 +6,34 @@ Description
 -----------
 
 COnTROT (COmprehensive Transcriptomic ORganizational Tool) is a program that
-will download and organize all expression data in GEO related to a search result,
-commonly an organism. This will identify and download GEO GDS, GSE, GSM, and GPL
-directories and files from NCBI FTP when provided with a downloaded GDS result
-from NCBI. Then the series data will be organized to retain matches to any 
+will download and organize all expression data in GEO related to a search result.
+This will identify and download GEO GDS, GSE, GSM, and GPL
+directories and files from NCBI FTP from a GDS search result
+at NCBI. Then the series data will be organized to retain matches to any 
 gene annotation present in the GenBank file provided. The data will be
 organized, mean centered, and then joined using the gene annotation into a
 single text file that can be easily manipulated or opened in Excel.
 
-To generate the initial file, perform a search in NCIB for GEO DataSets. Then in the
-upper right hand corner of the search results, select 'Send to:' and choose
-'File' and then click 'Create File'. This will contain a summary of your search
-results. This script will search this file for series indicators and download
-the respective files and directories. You will also need the GenBank file for your organism.
+The results will be two files: one with the data mean centered and the other with no mean centering.
+
+The user must provide the GenBank file to use for annotation from NCBI.
+
+COnTORT will request the search term and use that to search NCBI GEO and download the NCBI GEO results.
+
+Note that COnTORT can only download and combine normalized and processed data imported into NCBI GEO.
 
 Notes
 -----
 
     Must be run on scarcity. Requires Python3
 
-    inputs : A gds_result text file downloaded from NCBI.
-             The GenBank (.gbff) for your organism, downloaded from NCBI.
+    inputs : The GenBank (.gbff) for your organism, downloaded from NCBI.
     
     output : Primary output is a single text file containing the gene IDs on the first column
-    and the mean centered expression data present in the remaining columns. The first row
+    and the organized expression data present in the remaining columns. The first row
     contains the headers references the specific GEO IDs for each experiment. 
     Note that annotations no present in the GenBank file are listed with "N/A".
+    There will be two files:  one where the results are mean-centered for each experiment and one where no mean-centering is performed.
     
         Locus_Tag    Old_Locus_Tag    Gene_Name    Gene_Synonyms    Product    GSM_ID_1    GSM_ID_2
         RSP_0002     N/A              spbB         N/A              H-NS       12.0         4.0
@@ -40,58 +42,61 @@ Notes
         
     Directories created:
         - geo                     - the downloaded GDS, GSE, GSM and GPL directories
-        - matrix_files            - the series_matrix.txt files downloaded from GEO
-        - GEOquery_results        - the results of the GEOquery.r script
-        - GFF_match_output        - the results of matching the GEOquery results to the gene IDs from the GFF
+        - GEOannotate_results     - the results of running GEOparse to organize the annotation and data
+        - GeneOrf_match_output    - the results of matching the GEOparse results to the gene IDs from the GFF
         - mean_centered_results   - the mean centered expression data for each experiment
         - FTP_files               - files used to download the data from GEO via FTP
         - log_files               - all log files from each step as well as other saved files
         
     method :
-        Step 1) The user submitted GDSfile is parsed into two new files
+        Step 1) The NCBI GEO search result from the term provided by the user is downloaded to a GDS file and is parsed into two new files
     
         Step 2) The GEO data from the GDSfile is downloaded to the directory via FTP
     
-        Step 3) GEOquery.r is run to organize the GEO Series data
+        Step 3) GEOParse is run to organize the GEO data for which normalized data is available in NCBI GEO
     
         Step 4) The organized GEO Series file is matched to
-                the gene annotation information and the resulting data are mean
-                centered within each experiment and combined into the final output file.
+                the gene annotation information and the resulting data are or are not mean
+                centered within each experiment and combined into the final output files.
         
         Step 5) Clean up and orgainze the directory
     
     dependencies : 
         - python 3
-        - R libraries GEOquery and tidyverse
-        - Python modles argparse, ftplib, functools, glob, gzip, io, os, pandas, re, shutil, subprocess, sys, time
-        - GEO_annotate.r and GEOpatch.r Rscript in the same directory as COnTROT.py
+        - Python modules argparse, Bio, ftplib, functools, GEOparse, os, pandas, re, shutil, sys, subprocess, sys, tkinter, time, urllib
 
     usage [standard]:
-        python3.6 CONTORT.py -f gds_result.txt -gb genbank.gbff
+        python CONTORT.py
+    
+    usage [from pip install]:
+        contort
                 
-        run with nohup and & to prevent timeouts
 """
 # Import required Python modules
 import argparse
 from Bio import SeqIO
+from Bio import Entrez
 import ftplib
 import GEOparse
 import os
+from os import path
 import pandas as pd
 from pathlib import Path
 import re
 import shutil
 import sys
+import tkinter as tk
+from tkinter import messagebox, Label, Button, Entry, W, Tk, Toplevel, filedialog, ttk
 import time
-import urllib.request
+import urllib.request as request
 
 
 def runFindGEOAddresses( GDSfile ):
     """
     Parse GEOfile
     
-    Opens and parses the GDSfile result txt file provided by the user (downloaded
-    from NCBI).
+    Opens and parses the GDSfile result txt file from the result of the search term
+    provided by the user in the program.
     
     Creates new files:
         GEO_FTP_Addresses.txt - all GDS, GSE, GSM, GPL addresses in the file
@@ -205,14 +210,13 @@ def run_download_ftp_file(ftp_handle, name, dest, overwrite):
             with open('FTP_download_log.txt','a') as log:
                 log.write("downloaded: {0}\n".format(dest)) # write log file
                 log.close()
-        except FileNotFoundError:                      # error warning
+        except:                      # error warning
             with open('FTP_download_log.txt','a') as log:
                 log.write("FAILED: {0}".format(dest)) # write error in log file
                 log.close()
     else:
         print("already exists: {0}".format(dest))
 
-
 def run_mirror_ftp_dir(ftp_handle, name, overwrite, guess_by_extension):
     """
     Replicate the directories
@@ -220,13 +224,15 @@ def run_mirror_ftp_dir(ftp_handle, name, overwrite, guess_by_extension):
     """
     
     for item in ftp_handle.nlst(name):
-        if run_is_ftp_dir(ftp_handle, item, guess_by_extension):
-            run_mirror_ftp_dir(ftp_handle, item, overwrite, guess_by_extension)
-        else:
-            run_download_ftp_file(ftp_handle, item, item, overwrite)
+        try:
+            if run_is_ftp_dir(ftp_handle, item, guess_by_extension):
+                run_mirror_ftp_dir(ftp_handle, item, overwrite, guess_by_extension)
+            else:
+                run_download_ftp_file(ftp_handle, item, item, overwrite)
+        except:
+            continue
 
-
-def download_ftp_tree(ftp_handle, path, destination, overwrite=False, guess_by_extension=True):
+def download_ftp_tree(ftp_handle, FTPpath, destination, overwrite=False, guess_by_extension=True):
     """
     Performs the actions to download files from the NCBI FTP
     
@@ -235,15 +241,20 @@ def download_ftp_tree(ftp_handle, path, destination, overwrite=False, guess_by_e
     Will NOT overwrite files if present in the local directory
     """
    
-    path = path.lstrip("/")
+    path = FTPpath.lstrip("/")
     original_directory = os.getcwd()    # remember working directory before function is executed
     os.chdir(destination)               # change working directory to ftp mirror directory
-    run_mirror_ftp_dir(ftp_handle, path, overwrite, guess_by_extension)
+    try:
+        run_mirror_ftp_dir(ftp_handle, path, overwrite, guess_by_extension)
+    except:
+        with open('FTP_error_log.txt', 'a') as log:
+            log.write(f"{path}")
+            log.close()
     os.chdir(original_directory)        # reset working directory to what it was before function exec
 
 def GEOannotate():
     """
-    This is a replacement for R and GEOquery. This will organize the GEO data available.
+    This will organize the GEO data available using GEOparse.
     
     This first finds and copies all soft.gz files for all GSE (series) data in the 
     GEO download from the previous steps. Using GEOparse, the metadata for each gene is 
@@ -264,38 +275,25 @@ def GEOannotate():
     
     soft_files = [ fn for fn in os.listdir(os.getcwd()) if fn.endswith("soft.gz") ] 
     
-    series = []
-    
-    # Open directories file and get GSE entires. Write to list
-    
-    with open('GEO_FTP_Directories.txt', 'r') as f:
-        for line in f:
-            series_name = line.rstrip().split('/')[4]
-            if series_name.startswith("GDS"):
-                continue
-            if series_name.startswith("GSE"):
-                series.append(series_name)
+    for each in soft_files: #need to work on skipping "each" if there is a keyError
+        try:
+            gse = GEOparse.get_GEO(filepath=each)
+            samples = gse.pivot_samples("VALUE")
+            samples.reset_index(level = 0, inplace = True)
+            samples.rename(columns={'ID_REF':"ID"}, inplace = True)        
             
-    #with open('series.txt', 'a+') as f:
-     #   for row in series:
-      #      f.write("%s\n" % str(row)) #save series entires to file
+            for key,value in gse.gpls.items():
+                GPL = key
+            annotation = gse.gpls[GPL].table
+            annotation.reset_index(level = 0, inplace = True)
+    
+            result = pd.merge(annotation, samples)
+            gseID = each.split("_")[0]
+            output = gseID + "_GEOannotate_results.txt"
+            result.to_csv(output, sep = "\t", index = False)
+        except:
+            pass
 
-    for each in soft_files:
-        gse = GEOparse.get_GEO(filepath=each)
-        samples = gse.pivot_samples("VALUE")
-        samples.reset_index(level = 0, inplace = True)
-        samples.rename(columns={'ID_REF':"ID"}, inplace = True)
-        
-        for key,value in gse.gpls.items():
-            GPL = key
-        annotation = gse.gpls[GPL].table
-        annotation.reset_index(level = 0, inplace = True)
-        
-        result = pd.merge(annotation, samples)
-        gseID = each.split("_")[0]
-        output = gseID + "_GEOannotate_results.txt"
-        result.to_csv(output, sep = "\t", index = False)
-        
     for each in soft_files:
         os.remove(each)
 
@@ -435,7 +433,79 @@ def gffMatch( GBFF ):
                         
     GEO_output_results = [ fn for fn in os.listdir(os.getcwd()) if fn.endswith("output_results.txt") ]
 
+    # Combine and write non-mean centered results file
+
+    # Rename any results file that is less than 1 MB (no data)   
+
+    for each in GEO_output_results:
+        if os.path.getsize(each) < 1000:
+            name = each.split('_')[0]
+            os.rename(each, name+"_output_results_small.txt")
+            
+    GEO_output_results2 = [ fn for fn in os.listdir(os.getcwd()) if fn.endswith("output_results.txt") ]
+    
+    # Remove any duplicate values that appear in the mean center results files
+    
+    for f in GEO_output_results2:
+        df_name = f.split("_")[0]
+        df = pd.read_csv(f, sep = "\t")
+        df.sort_values("Experiment_ORF", inplace = True)
+        df2 = df.drop_duplicates(subset = "Experiment_ORF", keep = 'first')
+        outname = df_name + "_output_results_noDuplicates.txt"
+        df2.to_csv(outname, sep = "\t", header = True, index = False)
+        
+    GEO_output_results_noDups = [ fn for fn in os.listdir(os.getcwd()) if fn.endswith("output_results_noDuplicates.txt") ]
+    
+    # Combine all the results files into the final data file
+    
+    outputCombo = {}  
+    outputCombo['Locus_Tag\tOld_Locus_Tag\tGene_Name\tGene_Synonyms\tProduct'] = []
+    for each in GEO_output_results_noDups:    
+        with open(each, 'r') as g:
+            firstLine = g.readlines()
+            header = firstLine[0]
+            header2 = header.rstrip('\n').split('\t')[1:] 
+            outputCombo['Locus_Tag\tOld_Locus_Tag\tGene_Name\tGene_Synonyms\tProduct'].append(header2)
+            for each in firstLine:
+                name2 = each.split('\t')[0]
+                data = each.rstrip('\n').split('\t')[1:]
+                if name2 in combo:
+                    name = str(combo[name2])
+                    if name not in outputCombo:
+                        outputCombo[name]=[]
+                        outputCombo[name].append(data)
+                    else:
+                        outputCombo[name].append(data)
+                        
+    combined_output_final = []
+    
+    for key,val in outputCombo.items():
+        val2 = [val for sublist in outputCombo[key] for val in sublist]
+        val3 = "\t".join(val2)    
+        combined_output_final.append(f"{key}\t{val3}\n")
+        combined_output_final[1:] = sorted(combined_output_final[1:])
+    
+    combined_output_final2 = []    
+    for line in combined_output_final:
+        line2 = re.sub("\]", "", line)
+        line2 = re.sub("\[", "", line2)
+        line2 = re.sub("['']", "", line2)
+        line2 = line2.replace('\\t', '\t')
+        combined_output_final2.append(line2)
+
+    # Write COnTORT results file (Non-Mean Centered)
+        
+    final_original_output_name = "COnTORT_organized_non-mean_centered_transcriptomic_data.txt"
+    
+    with open(final_original_output_name, 'w') as f:
+        for each in combined_output_final2:
+            f.write(each)    
+    
     # Mean center the organized results for each experiment to make comparisons easier
+            
+    GEO_output_results = []
+    
+    GEO_output_results = [ fn for fn in os.listdir(os.getcwd()) if fn.endswith("output_results.txt") ]
     
     for each in GEO_output_results:
         geoName = each.split('_')[0]
@@ -456,7 +526,7 @@ def gffMatch( GBFF ):
     
     df_list_updated = [ fn for fn in os.listdir(os.getcwd()) if fn.endswith("meanCenter_results.txt") ]
 
-    # Remove any duplicate values that appear int he mean center results files
+    # Remove any duplicate values that appear in the mean center results files
         
     for f in df_list_updated:
         df_name = f.split("_")[0]
@@ -506,9 +576,9 @@ def gffMatch( GBFF ):
         line2 = line2.replace('\\t', '\t')
         combined_final2.append(line2)
 
-    # Write COnTORT results file
+    # Write COnTORT mean-centered results file
         
-    final_output_name = "COnTORT_organized_transcriptomic_data.txt"
+    final_output_name = "COnTORT_organized_mean_centered_transcriptomic_data.txt"
     
     with open(final_output_name, 'w') as f:
         for each in combined_final2:
@@ -521,20 +591,16 @@ def cleanUp( cwd ):
     Organize the files into folders for a cleaner directory
     """
     cwd = os.getcwd() + "/"    
-    # Organize downloaded series matrix files
-    os.mkdir( "matrix_files" )
-    originalContigDir = cwd + "/matrix_files/"
-    [ os.rename( (cwd + fn), (originalContigDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("series_matrix.txt") ]
-    [ os.rename( (cwd + fn), (originalContigDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("matrix.txt") ]
     # Organize GEOquery results files
     os.mkdir( "GEOannotate_results" )
-    gffDir = cwd + "/GEOquery_results/"
+    gffDir = cwd + "/GEOannotate_results/"
     [ os.rename( (cwd + fn), (gffDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("GEOannotate_results.txt") ]
     # Organize GFF match output result files
     os.mkdir( "GeneOrf_match_output" )
     inputContigsDir = cwd + "/GeneOrf_match_output/"
     [ os.rename( (cwd + fn), (inputContigsDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("output_results.txt") ]
-    [ os.rename( (cwd + fn), (gffDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("RPKM.txt") ]
+    [ os.rename( (cwd + fn), (inputContigsDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("output_results_small.txt") ]
+    [ os.rename( (cwd + fn), (inputContigsDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("output_results_noDuplicates.txt") ]
     # Organize mean centered result files
     os.mkdir( "mean_centered_results" )
     patserResultsDir = cwd + "/mean_centered_results/"
@@ -544,11 +610,7 @@ def cleanUp( cwd ):
     os.mkdir( "log_files" )
     patserResultsDir = cwd + "/log_files/"
     [ os.rename( (cwd + fn), (patserResultsDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("log.txt") ]
-    [ os.rename( (cwd + fn), (patserResultsDir + fn) ) for fn in os.listdir(cwd) if fn.startswith("series") ]
-    [ os.rename( (cwd + fn), (patserResultsDir + fn) ) for fn in os.listdir(cwd) if fn.startswith("gene_orf") ]
-    [ os.rename( (cwd + fn), (patserResultsDir + fn) ) for fn in os.listdir(cwd) if fn.endswith(".bt2") ]
-    [ os.rename( (cwd + fn), (patserResultsDir + fn) ) for fn in os.listdir(cwd) if fn.endswith(".fai") ]
-    [ os.rename( (cwd + fn), (patserResultsDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("sra_files.txt") ]
+    [ os.rename( (cwd + fn), (patserResultsDir + fn) ) for fn in os.listdir(cwd) if fn.endswith("gene_annotations.txt") ]
     # Organize GEO FTP files
     os.mkdir( "FTP_files" )
     patserResultsDir = cwd + "/FTP_files/"
@@ -557,65 +619,137 @@ def cleanUp( cwd ):
 def main():
     
     cmdparser = argparse.ArgumentParser(description="Find and Download NCBI files from GDS file downloaded from NCBI GEO, process the data, mean center. and combine the data by GenBank annotation.", usage='%(prog)s -f <gds_result.txt> -gb <genbank_file.gbff>', prog='CONTORT.py' )
-    cmdparser.add_argument('-f', '--file', action='store', dest='FILE', help = 'GDS_result.txt file downloaded from NCBI GEO.', metavar='')
-    cmdparser.add_argument('-gb', '--genbank', action='store', dest='GBFF', help = 'Genbank file for the organism of interest.', metavar='')
     cmdparser.add_argument('-d', '--detail',  action='store_true', dest='DETAIL',  help='Print a more detailed description of program.')
     cmdResults = vars(cmdparser.parse_args())
     cwd = os.getcwd()
     
     start = time.time() #start timer to time how long script takes to run
     
-    # if no args, print help
-    if len(sys.argv) == 1:
-        print("")
-        cmdparser.print_help()
-        sys.exit(1)
-            
     if cmdResults['DETAIL']:
-        print("\nScript Name: COnTORT.py")
-        print("\tThis script was designed to run on Scarcity by kmyers on 2019-05-14.")
-        print("\nPurpose : Bulk download of NCBI GEO Data from NCBI FTP. The data are then organized, combined with annotation from supplied GenBank files, mean centered, and concatenated.")
-        print("\nInput : A text file containing the GDS result file downloaded from NCBI.")
-        print("\nYou are encouranged to create a new directory in which to run this script.")
-        print("\nThis requires the R libraries GEOquery and tidyverse to be installed.")
-        print("Required Parameters: -f gds_result.txt\n")
-        print("Required Parameters: -gb genbank_file.gbff\n")
+        print("\nScript Name: COnTORT")
+        print("\tThis script was designed my Kevin Myers (kmyers2@wisc.edu).")
+        print("\nPurpose : Download NCBI GEO Data from NCBI FTP. The data are then organized, combined with annotation from supplied GenBank files, mean centered, and concatenated to a single file.")
+        print("\nInput : Using the GUI window, enter the search term for NCBI GEO and select the GenBank annotation file to use.")
+        print("\nThis requires the the following Python modules: argparse, Bio, ftplib, functools, GEOparse, os, pandas, re, shutil, sys, subprocess, sys, tkinter, time, urllib.")
         print("\tDEFAULT Server:  ftp.ncbi.nlm.nih.gov")
         print("\tDEFAULT User:  anonymous")
         print("\tDEFAULT password:  \n")
-        print("To run enter:  python3.6 COnTORT.py -f gds_result.txt -gb genbank_file.txt\n")
-        print("This will identify and download GEO GDS, GSE, GSM, and GPL directories and files from NCBI FTP when provided with a downloaded GDS result from NCBI. Then the series data will be organized to retain matches to the GenBank annotation (gene and ORF). The data will be organized, mean centered, and then joined using the gene annotations as IDs. \nTo generate the initial file, perform a search in NCIB for GEO DataSets. Then in the upper right hand corner of the search results, select 'Send to:' and choose 'File' and then click 'Create File'. This will contain a summary of your search results. This script will search this file for series indicators and download the respective files and directories.\n")
+        print("To run enter: contort\n")
+        print("Then use the window that appears to enter the required inforamtion\n")
         print("See Kevin Myers (kmyers2@wisc.edu) for problems with COnTORT.\n\n")
         sys.exit(1)
         
-    if cmdResults['FILE'] is not None:
-        GDSfile = cmdResults['FILE']
-    
-    if cmdResults['GBFF'] is not None:
-        GBFF = cmdResults['GBFF']
-        
-    if urllib.request.urlopen('http://www.google.com').getcode() == 0:
-        print("\nUh oh! It looks like you don't have access to the interet. CoNTORT requires internet access to function. Please try again later when you have access to the internet.\n")
-    else:
-        pass
-    
-    if urllib.request.urlopen('http://www.apple.com').getcode() == 0:
-        print("\nUh oh! It looks like you don't have access to the interet. CoNTORT requires internet access to function. Please try again later when you have access to the internet.\n")
-    else:
-        pass
-    
-    if urllib.request.urlopen('https://www.ncbi.nlm.nih.gov/geo').getcode() == 0:
-        print("\nUh oh! It looks like NCBI GEO is down. COnTORT cannot work without access to NCBI GEO. Please try again later when the webiste is available.\n")
-    else:
-        pass
-    
-    if urllib.request.urlopen('https://www.ncbi.nlm.nih.gov/sra').getcode() == 0:
-        print("\nUh oh! It looks like NCBI SRA is down. COnTORT cannot work without access to NCBI SRA. Please try again later when the webiste is available.\n")
-    else:
-        pass
-    
     print("\nThanks for using COnTORT! Let's get started...\n")
     
+    #TKinter class to make it easier for the user to enter the NCBI GEO search term and select the GenBank file to use
+    
+    class GetInfo:
+        def __init__(self, master):
+            self.master = master
+            master.title("COnTORT")
+            self.GenBank_File = None
+            self.file_only = None
+            
+            self.label = Label(master, text = "Please enter the information for COnTORT to run.", font=("Arial Bold", 15))
+            self.label.grid(column=0, row=0)
+            
+            self.GEOsearch_label = Label(master, text = "GEO Search Term:", font=("Arial Bold", 13)).grid(row = 1, column = 0, sticky = W)
+            self.GEOsearch_explanation = Label(master, text = "Enter the phrase above you would like to search NCBI GEO for experiments in").grid(row = 3, column = 0, sticky = W)
+            self.GEOsearch_explanation2 = Label(master, text = "High-throughput RNA Sequencing, Expression Microarrays, and Tiled Expression Microarrays.").grid(row = 4, column = 0, sticky = W)
+            self.GEOsearch_explanation3 = Label(master, text = "COnTORT will analyze all available data on NCBI GEO matching the search results.").grid(row = 5, column = 0, sticky = W)
+            self.GEOsearch_explanation3 = Label(master, text = "The more specific search term, the fewer results will be obtained.").grid(row = 6, column = 0, sticky = W)
+            self.GEOsearch_explanation4 = Label(master, text = "  ").grid(row = 7, column = 0, sticky = W)
+            self.GEOsearch_explanation5 = Label(master, text = "  ").grid(row = 8, column = 0, sticky = W)
+
+            self.GenBankFileLabel = Label(master, text = "Open the GenBank File to Use", font=("Arial Bold", 13)).grid(row = 9, column = 0, sticky = W)
+            self.GEO_term = Entry(master, width = 48)
+            openGenBankFileCommand = master.register(self.GenBankFileOpen)
+            self.genbank_button = ttk.Button(master, text = "Select GenBank File",
+                                         command = openGenBankFileCommand).grid(row=10, column = 0, sticky = W, pady=10)
+            self.GenBank_explanation = Label(master, text = "Use the button above to open the GenBank file (.gbff) for the annotation you would like to use in COnTORT.").grid(row = 11, column = 0, sticky = W)
+            self.GenBank_explanation2 = Label(master, text = "GenBank annotation files can be downloaded for the specific strain and species").grid(row = 12, column = 0, sticky = W)
+            self.GenBank_explanation3 = Label(master, text = "from https://www.ncbi.nlm.nih.gov/genome/.").grid(row = 13, column = 0, sticky = W)
+            self.GenBank_explanation3 = Label(master, text = "Use the GenBank annotation file to match the NCBI GEO search of interest.").grid(row = 14, column = 0, sticky = W)
+            self.GEOsearch_explanation4 = Label(master, text = "  ").grid(row = 15, column = 0, sticky = W)
+            self.GEOsearch_explanation5 = Label(master, text = "  ").grid(row = 16, column = 0, sticky = W)
+            
+            self.GEO_term.grid(row = 1, column = 0)
+            
+            runContortCommand = master.register(self.getInput)
+            exitContortCommand = master.register(self.getCancel)
+            
+            self.run_button=ttk.Button(master, text = "Run",
+                       command = runContortCommand).grid(row = 20, column = 0, sticky = W)
+            
+            self.cancel_button=ttk.Button(master, text = "Cancel",
+                       command = exitContortCommand).grid(row = 20, column = 1, sticky = W)
+            
+        def GenBankFileOpen(self):
+            filename = filedialog.askopenfilename(title = "Open the GenBank file", filetypes = (("GenBank Files", "*.gbff"),("All Files", "*")))
+            if filename: 
+                self.GenBank_File = filename
+                self.file_only = self.GenBank_File.split('/')[-1]
+            
+        def getInput(self):
+            self.GEO_Search_Term = self.GEO_term.get()
+            #self.GenBank_File_use = self.GenBank_File
+            #self.GenBank_File_use
+            if not self.GenBank_File:
+                self.warning_window = tk.messagebox.showerror('Error', 'Please select a GenBank file to use.')
+            else:
+                self.close_box_window = tk.messagebox.askokcancel('Running COnTORT', "COnTORT will search and download the results for the following search term:\n{}\n\nCOnTORT will use the GenBank file: {}.".format(self.GEO_term.get(), self.file_only), default = 'ok')
+                if self.close_box_window == True:
+                    root.destroy()
+                else:
+                    return
+        
+        def getCancel(self):
+            
+            self.MsgBox_window = tk.messagebox.askokcancel("Exit COnTORT", "Are you sure you want to exit COnTORT?", icon = "warning", default = 'cancel')
+            if self.MsgBox_window == False:
+                return
+            else:
+                root.destroy()
+
+    #Use TKinter to open a window to enter the NCBI GEO Search Term and select the GenBank File to use.
+
+    root = Tk()
+    root.geometry('800x500')
+    contort_gui = GetInfo(root)
+
+    root.mainloop()
+    
+    GDSsearch = contort_gui.GEO_Search_Term
+    try:
+        GBFF = contort_gui.file_only
+    except:
+        exit
+    
+    #TO GENERATE THE GDS RESULT FILE USING ENTREZ from BIOPYTHON:
+
+    Entrez.email='kmyers2@wisc.edu'
+    Entrez.api_key = "c47154faf0e8e818a313b184dc893b7b5a08"
+    idlist = []
+    handle = Entrez.esearch(db="gds", term=GDSsearch, retmax = '100000')
+    result = Entrez.read(handle)
+    for each in result['IdList']:
+        idlist.append(each)
+    for each in idlist:
+        test = Entrez.esummary(db="gds", id=each)
+        record = Entrez.read(test)
+        for entry in record:
+            if "Expression profiling by array" in entry['gdsType']:
+                with open('GDS_result.txt', 'a') as f:
+                    f.write(f"{entry['title']}\n{entry['summary']}\nOrganism:\t{entry['taxon']}\nType:\t{entry['gdsType']}\nPlatform:\tGPL{entry['GPL']}\t\t{entry['n_samples']} Samples\nFTP download:\t{entry['FTPLink']}\nSeries\t\tAccession:\t{entry['Accession']}\tID:\t{entry['Id']}\n\n")
+            elif "Expression profiling by high throughput sequencing" in entry['gdsType']:
+                with open('GDS_result.txt', 'a') as f:
+                    f.write(f"{entry['title']}\n{entry['summary']}\nOrganism:\t{entry['taxon']}\nType:\t{entry['gdsType']}\nPlatform:\tGPL{entry['GPL']}\t\t{entry['n_samples']} Samples\nFTP download:\t{entry['FTPLink']}\nSeries\t\tAccession:\t{entry['Accession']}\tID:\t{entry['Id']}\n\n")
+            elif "Expression profiling by genome tiling array" in entry['gdsType']:
+                with open('GDS_result.txt', 'a') as f:
+                    f.write(f"{entry['title']}\n{entry['summary']}\nOrganism:\t{entry['taxon']}\nType:\t{entry['gdsType']}\nPlatform:\tGPL{entry['GPL']}\t\t{entry['n_samples']} Samples\nFTP download:\t{entry['FTPLink']}\nSeries\t\tAccession:\t{entry['Accession']}\tID:\t{entry['Id']}\n\n")    
+
+    GDSfile = 'GDS_result.txt'
+    
     runFindGEOAddresses( GDSfile )
         
     server = 'ftp.ncbi.nlm.nih.gov'
@@ -626,12 +760,13 @@ def main():
       
     with open('GEO_FTP_Directories.txt', 'r') as f:
         for line in f:
-            path = line.rstrip()
-            download_ftp_tree(ftp, path, destination)
+            FTPpath = line.rstrip()
+            download_ftp_tree(ftp, FTPpath, destination)
 
     print("Organizing and combining the transcriptomic data downloaded from NCBI GEO...\n") 
     
-    #GEOquery()
+    GBFF = contort_gui.file_only
+    
     GEOannotate()
     gffMatch( GBFF )
     cleanUp( cwd )
@@ -641,8 +776,8 @@ def main():
     total_time = round(end - start, 2)
     total_time_min = round(total_time/60, 2)
     total_time_hours = round(total_time/60/60, 2)
-    print(f"It took {total_time_hours} hours ({total_time_min} minutes) to download, organize, and combine the data.\n")
+    print(f"\n\nIt took {total_time_hours} hours ({total_time_min} minutes) to download, organize, and combine the data.\n")
     print(f"We hope you enjoyed using COnTORT! Please e-mail Kevin Myers (kmyers2@wisc.edu) if you have any questions.\n")
-                           
+
 if __name__ == "__main__":
     main()
\ No newline at end of file
